from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class GeoInfo(BaseModel):
    country: Optional[str] = None
    city: Optional[str] = None


GeoInfo.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.offer_template_content_field import OfferTemplateContentField
from pianosdk.publisher.models.template_user_model import TemplateUserModel
from typing import List


class OfferTemplateVariant(BaseModel):
    offer_template_variant_id: Optional[str] = None
    offer_template_id: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    live_thumbnail_image_url: Optional[str] = None
    deleted: Optional[bool] = None
    create_date: Optional[datetime] = None
    create_by: Optional['TemplateUserModel'] = None
    update_date: Optional[datetime] = None
    update_by: Optional['TemplateUserModel'] = None
    archived_date: Optional[datetime] = None
    archived_by: Optional['TemplateUserModel'] = None
    status: Optional[str] = None
    content_field_list: Optional['List[OfferTemplateContentField]'] = None
    is_inherited: Optional[bool] = None


OfferTemplateVariant.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class GlobalTemplateDeploymentItem(BaseModel):
    item_id: Optional[str] = None
    template_id: Optional[str] = None
    aid: Optional[str] = None
    status: Optional[str] = None


GlobalTemplateDeploymentItem.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.template_version import TemplateVersion
from typing import List


class AdvancedEmailModel(BaseModel):
    email_id: Optional[int] = None
    name: Optional[str] = None
    caption: Optional[str] = None
    description: Optional[str] = None
    publisher_config: Optional[str] = None
    xdays: Optional[int] = None
    default_xdays: Optional[int] = None
    template_versions: Optional['List[TemplateVersion]'] = None
    default_template_id: Optional[str] = None
    system_template_id: Optional[str] = None
    caption_localized: Optional[str] = None
    description_localized: Optional[str] = None


AdvancedEmailModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class GracePeriodDetails(BaseModel):
    in_grace_period: Optional[bool] = None
    grace_period_start_date: Optional[datetime] = None
    grace_period_end_date: Optional[datetime] = None


GracePeriodDetails.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.subscription_reallocation_error import SubscriptionReallocationError
from typing import List


class SubscriptionReallocationValidationResult(BaseModel):
    subscription_reallocation_errors: Optional['List[SubscriptionReallocationError]'] = None


SubscriptionReallocationValidationResult.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class Export(BaseModel):
    export_id: Optional[str] = None
    export_name: Optional[str] = None
    export_created: Optional[datetime] = None
    export_completed: Optional[datetime] = None
    export_percentage: Optional[int] = None
    export_records: Optional[int] = None
    export_status: Optional[str] = None
    report_type: Optional[str] = None
    export_updated: Optional[datetime] = None
    export_repeatable: Optional[bool] = None
    filter_data: Optional[str] = None


Export.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.period import Period
from typing import List


class Schedule(BaseModel):
    aid: Optional[str] = None
    name: Optional[str] = None
    schedule_id: Optional[str] = None
    deleted: Optional[bool] = None
    create_date: Optional[datetime] = None
    update_date: Optional[datetime] = None
    periods: Optional['List[Period]'] = None


Schedule.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PaySourceDTO(BaseModel):
    id: Optional[int] = None
    identifier: Optional[str] = None
    caption: Optional[str] = None
    title: Optional[str] = None
    custom_title: Optional[str] = None


PaySourceDTO.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.export import Export


class PublisherExportCreateAccessReportExportApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def create_access_report_export_tz(self, aid: str, export_name: str, date_from: datetime, date_to: datetime, access_status: str = 'ALL', term_type: List[str] = None, term_id: List[str] = None, next_billing_date: datetime = None, last_payment_status: str = None, end_date_from: datetime = None, end_date_to: datetime = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/accessReportExport/v2'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'access_status': _encode_parameter(access_status),
            'term_type': _encode_parameter(term_type),
            'term_id': _encode_parameter(term_id),
            'next_billing_date': _encode_parameter(next_billing_date),
            'last_payment_status': _encode_parameter(last_payment_status),
            'date_from': _encode_parameter(date_from),
            'date_to': _encode_parameter(date_to),
            'end_date_from': _encode_parameter(end_date_from),
            'end_date_to': _encode_parameter(end_date_to)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class GeoLocation(BaseModel):
    region_code: Optional[str] = None
    region_name: Optional[str] = None
    city: Optional[str] = None
    country_code: Optional[str] = None
    postal_code: Optional[str] = None


GeoLocation.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.id.models.geo_info import GeoInfo


class Session(BaseModel):
    aid: Optional[str] = None
    uid: Optional[str] = None
    jti: Optional[str] = None
    location: Optional['GeoInfo'] = None
    creation_date: Optional[int] = None
    last_activity_date: Optional[int] = None
    user_agent: Optional[str] = None
    trackable: Optional[bool] = None


Session.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class WebhookConfig(BaseModel):
    key: Optional[str] = None
    label: Optional[str] = None
    enabled: Optional[bool] = None
    type: Optional[str] = None


WebhookConfig.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UpgradeOfferOptionModel(BaseModel):
    term_change_option_id: Optional[str] = None
    description: Optional[str] = None
    from_term_id: Optional[str] = None
    from_term_name: Optional[str] = None
    from_period_id: Optional[str] = None
    from_period_name: Optional[str] = None
    from_billing_plan: Optional[str] = None
    to_term_id: Optional[str] = None
    to_term_name: Optional[str] = None
    to_period_id: Optional[str] = None
    to_period_name: Optional[str] = None
    to_billing_plan: Optional[str] = None
    billing_timing: Optional[str] = None
    immediate_access: Optional[str] = None
    prorate_access: Optional[str] = None


UpgradeOfferOptionModel.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class WebhookResponse(BaseModel):
    status: Optional[str] = None
    status_localized: Optional[str] = None
    response_headers: Optional[str] = None
    response_body: Optional[str] = None
    create_date: Optional[datetime] = None
    request_url: Optional[str] = None
    request_data: Optional[str] = None


WebhookResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.update_access_period_params import UpdateAccessPeriodParams
from typing import List


class UpdateBillingConfigurationRequest(BaseModel):
    periods: Optional['List[UpdateAccessPeriodParams]'] = None


UpdateBillingConfigurationRequest.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.id.models.session import Session
from typing import List


class UserSessionInfo(BaseModel):
    sessions: Optional['List[Session]'] = None
    total: Optional[int] = None


UserSessionInfo.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.id.models.social_account_detail import SocialAccountDetail
from typing import List


class SocialAccountDetailsResponse(BaseModel):
    social_accounts: Optional['List[SocialAccountDetail]'] = None


SocialAccountDetailsResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class Region(BaseModel):
    region_name: Optional[str] = None
    region_code: Optional[str] = None
    region_id: Optional[str] = None


Region.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.user_address import UserAddress


class Voucher(BaseModel):
    pub_id: Optional[str] = None
    code: Optional[str] = None
    state: Optional[str] = None
    state_label: Optional[str] = None
    recipient_name: Optional[str] = None
    recipient_email: Optional[str] = None
    recipient_message: Optional[str] = None
    send_date: Optional[datetime] = None
    create_date: Optional[datetime] = None
    expires: Optional[str] = None
    expire_date: Optional[datetime] = None
    redeemed: Optional[datetime] = None
    revoke_date: Optional[datetime] = None
    period: Optional[str] = None
    app_name: Optional[str] = None
    term_name: Optional[str] = None
    term_type: Optional[str] = None
    term_id: Optional[str] = None
    resource_name: Optional[str] = None
    price: Optional[str] = None
    transaction_id: Optional[str] = None
    is_revocable: Optional[bool] = None
    is_refundable: Optional[bool] = None
    is_resendable: Optional[bool] = None
    refund_amount: Optional[str] = None
    refund_amount_recalculated: Optional[bool] = None
    user_address: Optional['UserAddress'] = None


Voucher.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.id.models.consent_model import ConsentModel
from pianosdk.id.models.custom_field_value_dto import CustomFieldValueDto
from typing import Dict
from typing import List


class SocialAdditionalInputRequest(BaseModel):
    additional_input_state: Optional[str] = None
    consents: Optional['List[ConsentModel]'] = None
    custom_field_values: Optional['List[CustomFieldValueDto]'] = None
    email: Optional[str] = None
    aliases: Optional[Dict[str, str]] = None


SocialAdditionalInputRequest.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class SocialAccountDetail(BaseModel):
    provider_name: Optional[str] = None
    provider_user_id: Optional[str] = None
    user_email: Optional[str] = None


SocialAccountDetail.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.promotion import Promotion


class PublisherPromotionApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def code_exists(self, promotion_id: str, aid: str, email: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/promotion/exists'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'promotion_id': _encode_parameter(promotion_id),
            'aid': _encode_parameter(aid),
            'email': _encode_parameter(email)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def count(self, aid: str, expired: str = 'all') -> ApiResponse[int]:
        _url_path = '/api/v3/publisher/promotion/count'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'expired': _encode_parameter(expired)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, int)
        return _result

    def create(self, aid: str, name: str, new_customers_only: bool = False, start_date: datetime = None, end_date: datetime = None, discount_type: str = None, percentage_discount: float = None, unlimited_uses: bool = False, uses_allowed: int = None, never_allow_zero: bool = False, fixed_promotion_code: str = None, promotion_code_prefix: str = None, term_dependency_type: str = None, apply_to_all_billing_periods: bool = False, can_be_applied_on_renewal: bool = False, billing_period_limit: int = None) -> ApiResponse[Promotion]:
        _url_path = '/api/v3/publisher/promotion/create'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'name': _encode_parameter(name),
            'start_date': _encode_parameter(start_date),
            'end_date': _encode_parameter(end_date),
            'new_customers_only': _encode_parameter(new_customers_only),
            'discount_type': _encode_parameter(discount_type),
            'percentage_discount': _encode_parameter(percentage_discount),
            'unlimited_uses': _encode_parameter(unlimited_uses),
            'uses_allowed': _encode_parameter(uses_allowed),
            'never_allow_zero': _encode_parameter(never_allow_zero),
            'fixed_promotion_code': _encode_parameter(fixed_promotion_code),
            'promotion_code_prefix': _encode_parameter(promotion_code_prefix),
            'term_dependency_type': _encode_parameter(term_dependency_type),
            'apply_to_all_billing_periods': _encode_parameter(apply_to_all_billing_periods),
            'can_be_applied_on_renewal': _encode_parameter(can_be_applied_on_renewal),
            'billing_period_limit': _encode_parameter(billing_period_limit)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Promotion)
        return _result

    def delete(self, promotion_id: str, aid: str) -> ApiResponse[Dict]:
        _url_path = '/api/v3/publisher/promotion/delete'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'promotion_id': _encode_parameter(promotion_id),
            'aid': _encode_parameter(aid)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response)
        return _result

    def generate_codes(self, promotion_id: str, aid: str, fixed_promotion_code: str = None, promotion_code_prefix: str = None, amount: int = None, asset_id: str = None) -> ApiResponse[Promotion]:
        _url_path = '/api/v3/publisher/promotion/generate'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'promotion_id': _encode_parameter(promotion_id),
            'aid': _encode_parameter(aid),
            'fixed_promotion_code': _encode_parameter(fixed_promotion_code),
            'promotion_code_prefix': _encode_parameter(promotion_code_prefix),
            'amount': _encode_parameter(amount),
            'asset_id': _encode_parameter(asset_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Promotion)
        return _result

    def get(self, promotion_id: str, aid: str) -> ApiResponse[Promotion]:
        _url_path = '/api/v3/publisher/promotion/get'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'promotion_id': _encode_parameter(promotion_id),
            'aid': _encode_parameter(aid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Promotion)
        return _result

    def list(self, aid: str, offset: int = 0, limit: int = 100, expired: str = 'active', order_by: str = None, order_direction: str = None, q: str = None) -> ApiResponse[List[Promotion]]:
        _url_path = '/api/v3/publisher/promotion/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'expired': _encode_parameter(expired),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'q': _encode_parameter(q),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Promotion)
        return _result

    def total_sale(self, promotion_id: str, aid: str, currency_code: str = 'USD') -> ApiResponse[str]:
        _url_path = '/api/v3/publisher/promotion/total'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'promotion_id': _encode_parameter(promotion_id),
            'aid': _encode_parameter(aid),
            'currency_code': _encode_parameter(currency_code)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, str)
        return _result

    def update(self, promotion_id: str, aid: str, name: str, discount_type: str, start_date: datetime = None, end_date: datetime = None, new_customers_only: bool = False, percentage_discount: float = None, unlimited_uses: bool = False, uses_allowed: int = None, never_allow_zero: bool = False, fixed_promotion_code: str = None, promotion_code_prefix: str = None, term_dependency_type: str = None, apply_to_all_billing_periods: bool = None, can_be_applied_on_renewal: bool = False, billing_period_limit: int = None) -> ApiResponse[Promotion]:
        _url_path = '/api/v3/publisher/promotion/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'promotion_id': _encode_parameter(promotion_id),
            'aid': _encode_parameter(aid),
            'name': _encode_parameter(name),
            'start_date': _encode_parameter(start_date),
            'end_date': _encode_parameter(end_date),
            'new_customers_only': _encode_parameter(new_customers_only),
            'discount_type': _encode_parameter(discount_type),
            'percentage_discount': _encode_parameter(percentage_discount),
            'unlimited_uses': _encode_parameter(unlimited_uses),
            'uses_allowed': _encode_parameter(uses_allowed),
            'never_allow_zero': _encode_parameter(never_allow_zero),
            'fixed_promotion_code': _encode_parameter(fixed_promotion_code),
            'promotion_code_prefix': _encode_parameter(promotion_code_prefix),
            'term_dependency_type': _encode_parameter(term_dependency_type),
            'apply_to_all_billing_periods': _encode_parameter(apply_to_all_billing_periods),
            'can_be_applied_on_renewal': _encode_parameter(can_be_applied_on_renewal),
            'billing_period_limit': _encode_parameter(billing_period_limit)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Promotion)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import Dict
from typing import List


class ImportCFResult(BaseModel):
    code: Optional[int] = None
    ts: Optional[int] = None
    data: Optional['Dict[str, List[Dict[str, str]]]'] = None


ImportCFResult.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class AppResourceCount(BaseModel):
    aid: Optional[str] = None
    resource_count: Optional[int] = None


AppResourceCount.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ExtendedExperienceRevision(BaseModel):
    model: Optional[str] = None
    draft: Optional[str] = None
    draft_date: Optional[datetime] = None
    draft_base_version: Optional[int] = None


ExtendedExperienceRevision.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserAliasDto(BaseModel):
    type: Optional[str] = None
    value: Optional[str] = None
    display_value: Optional[str] = None


UserAliasDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.licensee_manager import LicenseeManager
from pianosdk.publisher.models.licensee_representative import LicenseeRepresentative
from typing import List


class Licensee(BaseModel):
    aid: Optional[str] = None
    licensee_id: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    logo_url: Optional[str] = None
    representatives: Optional['List[LicenseeRepresentative]'] = None
    managers: Optional['List[LicenseeManager]'] = None


Licensee.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TemplateVariable(BaseModel):
    template_variable_id: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    test_value: Optional[str] = None
    parent_id: Optional[str] = None


TemplateVariable.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserSubscriptionDto(BaseModel):
    term_name: Optional[str] = None
    term_id: Optional[str] = None
    type: Optional[str] = None
    payment_billing_plan: Optional[str] = None
    billing_plan: Optional[str] = None
    image_url: Optional[str] = None
    resource_name: Optional[str] = None
    rid: Optional[str] = None
    next_bill_date: Optional[str] = None
    subscription_last_payment: Optional[str] = None
    status: Optional[str] = None
    status_label: Optional[str] = None
    creadit_card_expire: Optional[str] = None
    creadit_card_expire_soon: Optional[bool] = None
    subscription_id: Optional[str] = None
    payment_method: Optional[str] = None
    access_expired: Optional[bool] = None
    in_app_payment: Optional[bool] = None
    psc_subscriber_number: Optional[str] = None
    conversion_result: Optional[str] = None
    external_api_name: Optional[str] = None
    charge_count: Optional[int] = None
    status_display: Optional[str] = None
    auto_renew: Optional[bool] = None
    user_payment_info_id: Optional[str] = None
    access_id: Optional[str] = None
    last_conversion_id: Optional[str] = None
    last_payment_conversion_id: Optional[str] = None
    delivery_schedule_pub_id: Optional[str] = None


UserSubscriptionDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class SectionsCounters(BaseModel):
    section: Optional[str] = None
    template_count: Optional[int] = None
    section_name_localized: Optional[str] = None


SectionsCounters.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.export import Export


class PublisherExportCreateApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def create_access_report_export(self, aid: str, export_name: str, date_from: datetime, date_to: datetime, access_status: str = 'ALL', term_type: List[str] = None, term_id: List[str] = None, next_billing_date: datetime = None, last_payment_status: str = None, end_date_from: datetime = None, end_date_to: datetime = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/accessReportExport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'access_status': _encode_parameter(access_status),
            'term_type': _encode_parameter(term_type),
            'term_id': _encode_parameter(term_id),
            'next_billing_date': _encode_parameter(next_billing_date),
            'last_payment_status': _encode_parameter(last_payment_status),
            'date_from': _encode_parameter(date_from),
            'date_to': _encode_parameter(date_to),
            'end_date_from': _encode_parameter(end_date_from),
            'end_date_to': _encode_parameter(end_date_to)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result

    def create_daily_activity_report_export(self, aid: str, export_name: str, _date: datetime, term_type: List[str] = None, currency: str = None, currency_list: List[str] = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/dailyActivityReportExport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'date': _encode_parameter(_date),
            'term_type': _encode_parameter(term_type),
            'currency': _encode_parameter(currency),
            'currencyList': _encode_parameter(currency_list)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result

    def create_monthly_activity_report_export(self, aid: str, export_name: str, month: int, year: int, term_type: List[str] = None, currency: str = None, currency_list: List[str] = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/monthlyActivityReportExport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'month': _encode_parameter(month),
            'year': _encode_parameter(year),
            'term_type': _encode_parameter(term_type),
            'currency': _encode_parameter(currency),
            'currencyList': _encode_parameter(currency_list)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result

    def create_subscription_detailed_report(self, aid: str, export_name: str, q: str = None, search_new_subscriptions: bool = None, new_subscriptions_created_from: datetime = None, new_subscriptions_created_to: datetime = None, search_active_now_subscriptions: bool = None, active_now_subscriptions_statuses: List[str] = None, search_inactive_subscriptions: bool = None, inactive_subscriptions_statuses: List[str] = None, subscriptions_inactive_from: datetime = None, subscriptions_inactive_to: datetime = None, search_updated_subscriptions: bool = None, updated_subscriptions_statuses: List[str] = None, subscriptions_updated_from: datetime = None, subscriptions_updated_to: datetime = None, search_auto_renewing_subscriptions: bool = None, subscriptions_auto_renewing: bool = None, search_subscriptions_by_next_billing_date: bool = None, subscriptions_next_billing_date_from: datetime = None, subscriptions_next_billing_date_to: datetime = None, search_subscriptions_by_terms: bool = None, subscriptions_terms: List[str] = None, subscriptions_term_types: List[str] = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/subscriptionDetailsReport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'q': _encode_parameter(q),
            'search_new_subscriptions': _encode_parameter(search_new_subscriptions),
            'new_subscriptions_created_from': _encode_parameter(new_subscriptions_created_from),
            'new_subscriptions_created_to': _encode_parameter(new_subscriptions_created_to),
            'search_active_now_subscriptions': _encode_parameter(search_active_now_subscriptions),
            'active_now_subscriptions_statuses': _encode_parameter(active_now_subscriptions_statuses),
            'search_inactive_subscriptions': _encode_parameter(search_inactive_subscriptions),
            'inactive_subscriptions_statuses': _encode_parameter(inactive_subscriptions_statuses),
            'subscriptions_inactive_from': _encode_parameter(subscriptions_inactive_from),
            'subscriptions_inactive_to': _encode_parameter(subscriptions_inactive_to),
            'search_updated_subscriptions': _encode_parameter(search_updated_subscriptions),
            'updated_subscriptions_statuses': _encode_parameter(updated_subscriptions_statuses),
            'subscriptions_updated_from': _encode_parameter(subscriptions_updated_from),
            'subscriptions_updated_to': _encode_parameter(subscriptions_updated_to),
            'search_auto_renewing_subscriptions': _encode_parameter(search_auto_renewing_subscriptions),
            'subscriptions_auto_renewing': _encode_parameter(subscriptions_auto_renewing),
            'search_subscriptions_by_next_billing_date': _encode_parameter(search_subscriptions_by_next_billing_date),
            'subscriptions_next_billing_date_from': _encode_parameter(subscriptions_next_billing_date_from),
            'subscriptions_next_billing_date_to': _encode_parameter(subscriptions_next_billing_date_to),
            'search_subscriptions_by_terms': _encode_parameter(search_subscriptions_by_terms),
            'subscriptions_terms': _encode_parameter(subscriptions_terms),
            'subscriptions_term_types': _encode_parameter(subscriptions_term_types)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result

    def create_subscription_summary_report(self, aid: str, export_name: str, date_from: datetime, date_to: datetime) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/subscriptionSummaryReport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'date_from': _encode_parameter(date_from),
            'date_to': _encode_parameter(date_to)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result

    def create_term_change_report_export(self, aid: str, export_name: str, date_from: datetime = None, date_to: datetime = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/termChangeReportExport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'date_from': _encode_parameter(date_from),
            'date_to': _encode_parameter(date_to)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result

    def create_transactions_report(self, aid: str, export_name: str, transactions_type: str = 'purchases', order_by: str = 'payment_date', order_direction: str = 'desc', q: str = None, date_from: datetime = None, date_to: datetime = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/transactionsReport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'transactions_type': _encode_parameter(transactions_type),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'q': _encode_parameter(q),
            'date_from': _encode_parameter(date_from),
            'date_to': _encode_parameter(date_to)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result

    def create_user_export(self, aid: str, export_name: str, export_custom_fields: List[str] = None, export_all_custom_fields: bool = False, include_notes: bool = None, name: str = None, email: str = None, registered_from: datetime = None, registered_until: datetime = None, access_to_resources: List[str] = None, converted_terms: List[str] = None, access_from: datetime = None, access_until: datetime = None, converted_term_from: datetime = None, converted_term_until: datetime = None, converted_term_sharing_type: str = None, redeemed_promotions: List[str] = None, redeemed_promotion_from: datetime = None, redeemed_promotion_until: datetime = None, trial_period_is_active: bool = None, has_trial_period: bool = None, has_access: bool = None, has_conversion_term: bool = None, has_redeemed_promotion: bool = None, include_trial_redemptions: bool = None, converted_term_types: List[str] = None, has_conversion_term_type: bool = None, spent_money_currency: str = None, spent_money_from: float = None, spent_money_until: float = None, spent_from_date: datetime = None, spent_until_date: datetime = None, payment_methods: List[int] = None, billing_failure_from: datetime = None, billing_failure_until: datetime = None, had_billing_failure: bool = None, has_payment: bool = None, upi_ext_customer_id: str = None, credit_card_will_expire: str = None, active_subscription_to_resources: List[str] = None, has_active_subscription: bool = None, subscription_start_from: datetime = None, subscription_start_until: datetime = None, subscription_renew_from: datetime = None, subscription_renew_until: datetime = None, subscription_expire_from: datetime = None, subscription_expire_until: datetime = None, trial_expire_from: datetime = None, trial_expire_until: datetime = None, has_any_subscriptions: bool = None, has_subscription_will_renew: bool = None, has_subscription_will_expire: bool = None, has_subscription_starts: bool = None, has_unresolved_inquiry: bool = None, submitted_inquiry_from: datetime = None, submitted_inquiry_until: datetime = None, received_response_from: datetime = None, received_response_until: datetime = None, resolved_inquiry_from: datetime = None, resolved_inquiry_until: datetime = None, has_submitted_inquiry: bool = None, has_received_response_inquiry: bool = None, has_resolved_inquiry: bool = None, has_licensing_contract_redemptions: bool = None, selected_licensees: List[str] = None, selected_contracts: List[str] = None, licensing_contract_redeemed_from: datetime = None, licensing_contract_redeemed_until: datetime = None, data_type: List[str] = None, data: str = None, has_data: bool = None, has_last_access_time: bool = None, last_access_time_from: datetime = None, last_access_time_until: datetime = None, selected_consents_map: List[str] = None, consent_checked: bool = None, custom_fields: str = None, source: str = None, invert_credit_card_will_expire: bool = None, has_email_confirmation_required: bool = None, email_confirmation_state: str = None, consent_has_data: bool = None, q: str = None, order_by: str = None, order_direction: str = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/userExport'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'export_custom_fields': _encode_parameter(export_custom_fields),
            'export_all_custom_fields': _encode_parameter(export_all_custom_fields),
            'include_notes': _encode_parameter(include_notes),
            'name': _encode_parameter(name),
            'email': _encode_parameter(email),
            'registered_from': _encode_parameter(registered_from),
            'registered_until': _encode_parameter(registered_until),
            'access_to_resources': _encode_parameter(access_to_resources),
            'converted_terms': _encode_parameter(converted_terms),
            'access_from': _encode_parameter(access_from),
            'access_until': _encode_parameter(access_until),
            'converted_term_from': _encode_parameter(converted_term_from),
            'converted_term_until': _encode_parameter(converted_term_until),
            'converted_term_sharing_type': _encode_parameter(converted_term_sharing_type),
            'redeemed_promotions': _encode_parameter(redeemed_promotions),
            'redeemed_promotion_from': _encode_parameter(redeemed_promotion_from),
            'redeemed_promotion_until': _encode_parameter(redeemed_promotion_until),
            'trial_period_is_active': _encode_parameter(trial_period_is_active),
            'has_trial_period': _encode_parameter(has_trial_period),
            'has_access': _encode_parameter(has_access),
            'has_conversion_term': _encode_parameter(has_conversion_term),
            'has_redeemed_promotion': _encode_parameter(has_redeemed_promotion),
            'include_trial_redemptions': _encode_parameter(include_trial_redemptions),
            'converted_term_types': _encode_parameter(converted_term_types),
            'has_conversion_term_type': _encode_parameter(has_conversion_term_type),
            'spent_money_currency': _encode_parameter(spent_money_currency),
            'spent_money_from': _encode_parameter(spent_money_from),
            'spent_money_until': _encode_parameter(spent_money_until),
            'spent_from_date': _encode_parameter(spent_from_date),
            'spent_until_date': _encode_parameter(spent_until_date),
            'payment_methods': _encode_parameter(payment_methods),
            'billing_failure_from': _encode_parameter(billing_failure_from),
            'billing_failure_until': _encode_parameter(billing_failure_until),
            'had_billing_failure': _encode_parameter(had_billing_failure),
            'has_payment': _encode_parameter(has_payment),
            'upi_ext_customer_id': _encode_parameter(upi_ext_customer_id),
            'credit_card_will_expire': _encode_parameter(credit_card_will_expire),
            'active_subscription_to_resources': _encode_parameter(active_subscription_to_resources),
            'has_active_subscription': _encode_parameter(has_active_subscription),
            'subscription_start_from': _encode_parameter(subscription_start_from),
            'subscription_start_until': _encode_parameter(subscription_start_until),
            'subscription_renew_from': _encode_parameter(subscription_renew_from),
            'subscription_renew_until': _encode_parameter(subscription_renew_until),
            'subscription_expire_from': _encode_parameter(subscription_expire_from),
            'subscription_expire_until': _encode_parameter(subscription_expire_until),
            'trial_expire_from': _encode_parameter(trial_expire_from),
            'trial_expire_until': _encode_parameter(trial_expire_until),
            'has_any_subscriptions': _encode_parameter(has_any_subscriptions),
            'has_subscription_will_renew': _encode_parameter(has_subscription_will_renew),
            'has_subscription_will_expire': _encode_parameter(has_subscription_will_expire),
            'has_subscription_starts': _encode_parameter(has_subscription_starts),
            'has_unresolved_inquiry': _encode_parameter(has_unresolved_inquiry),
            'submitted_inquiry_from': _encode_parameter(submitted_inquiry_from),
            'submitted_inquiry_until': _encode_parameter(submitted_inquiry_until),
            'received_response_from': _encode_parameter(received_response_from),
            'received_response_until': _encode_parameter(received_response_until),
            'resolved_inquiry_from': _encode_parameter(resolved_inquiry_from),
            'resolved_inquiry_until': _encode_parameter(resolved_inquiry_until),
            'has_submitted_inquiry': _encode_parameter(has_submitted_inquiry),
            'has_received_response_inquiry': _encode_parameter(has_received_response_inquiry),
            'has_resolved_inquiry': _encode_parameter(has_resolved_inquiry),
            'has_licensing_contract_redemptions': _encode_parameter(has_licensing_contract_redemptions),
            'selected_licensees': _encode_parameter(selected_licensees),
            'selected_contracts': _encode_parameter(selected_contracts),
            'licensing_contract_redeemed_from': _encode_parameter(licensing_contract_redeemed_from),
            'licensing_contract_redeemed_until': _encode_parameter(licensing_contract_redeemed_until),
            'data_type': _encode_parameter(data_type),
            'data': _encode_parameter(data),
            'has_data': _encode_parameter(has_data),
            'has_last_access_time': _encode_parameter(has_last_access_time),
            'last_access_time_from': _encode_parameter(last_access_time_from),
            'last_access_time_until': _encode_parameter(last_access_time_until),
            'selected_consents_map': _encode_parameter(selected_consents_map),
            'consent_checked': _encode_parameter(consent_checked),
            'custom_fields': _encode_parameter(custom_fields),
            'source': _encode_parameter(source),
            'invert_credit_card_will_expire': _encode_parameter(invert_credit_card_will_expire),
            'has_emailConfirmation_required': _encode_parameter(has_email_confirmation_required),
            'email_confirmation_state': _encode_parameter(email_confirmation_state),
            'consent_has_data': _encode_parameter(consent_has_data),
            'q': _encode_parameter(q),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class EdgilPaywayHostedPageSetup(BaseModel):
    request_id: Optional[str] = None
    reply_url: Optional[str] = None
    action_url: Optional[str] = None


EdgilPaywayHostedPageSetup.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.upgrade_offer_option_model import UpgradeOfferOptionModel
from typing import List


class UpgradeOfferModel(BaseModel):
    aid: Optional[str] = None
    offer_id: Optional[str] = None
    name: Optional[str] = None
    upgrade_offer_options: Optional['List[UpgradeOfferOptionModel]'] = None


UpgradeOfferModel.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.shared_subscription import SharedSubscription


class PublisherSubscriptionShareApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def invite(self, aid: str, subscription_id: str, shared_account: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/subscription/share/invite'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'subscription_id': _encode_parameter(subscription_id),
            'shared_account': _encode_parameter(shared_account)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def list(self, aid: str, offset: int, limit: int, term_id: str = None, unused_accesses_only: bool = False, status: str = None, start_date: datetime = None, end_date: datetime = None, select_by: str = None) -> ApiResponse[List[SharedSubscription]]:
        _url_path = '/api/v3/publisher/subscription/share/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'offset': _encode_parameter(offset),
            'limit': _encode_parameter(limit),
            'term_id': _encode_parameter(term_id),
            'unused_accesses_only': _encode_parameter(unused_accesses_only),
            'status': _encode_parameter(status),
            'start_date': _encode_parameter(start_date),
            'end_date': _encode_parameter(end_date),
            'select_by': _encode_parameter(select_by)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, SharedSubscription)
        return _result

    def resend(self, aid: str, subscription_id: str, account_id: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/subscription/share/resend'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'subscription_id': _encode_parameter(subscription_id),
            'account_id': _encode_parameter(account_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def revoke(self, aid: str, subscription_id: str, account_id: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/subscription/share/revoke'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'subscription_id': _encode_parameter(subscription_id),
            'account_id': _encode_parameter(account_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def start(self, aid: str, subscription_id: str, shared_accounts: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/subscription/share/start'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'subscription_id': _encode_parameter(subscription_id),
            'shared_accounts': _encode_parameter(shared_accounts)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def stop(self, aid: str, subscription_id: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/subscription/share/stop'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'subscription_id': _encode_parameter(subscription_id)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def update(self, aid: str, subscription_id: str, shared_accounts: str) -> ApiResponse[bool]:
        _url_path = '/api/v3/publisher/subscription/share/update'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'subscription_id': _encode_parameter(subscription_id),
            'shared_accounts': _encode_parameter(shared_accounts)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, bool)
        return _result

    def validate(self, aid: str, rid: str, email: str) -> ApiResponse[str]:
        _url_path = '/api/v3/publisher/subscription/share/validate'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'rid': _encode_parameter(rid),
            'email': _encode_parameter(email)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, str)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.app import App
from typing import Any
from typing import Dict
from typing import List


class UserProviderConfiguration(BaseModel):
    user_provider_configuration_id: Optional[str] = None
    app: Optional['App'] = None
    source: Optional[str] = None
    title: Optional[str] = None
    app_id: Optional[str] = None
    app_name: Optional[str] = None
    client_secret: Optional[str] = None
    client_id: Optional[str] = None
    type_name: Optional[str] = None
    reset_password_client_id: Optional[str] = None
    reset_password_flow: Optional[str] = None
    reset_password_flow_version: Optional[str] = None
    reset_password_locale: Optional[str] = None
    reset_password_redirect_uri: Optional[str] = None
    reset_password_form: Optional[str] = None
    api_version: Optional[str] = None
    endpoint: Optional[str] = None
    api_key: Optional[str] = None
    api_secret: Optional[str] = None
    auth_key: Optional[str] = None
    site_code: Optional[str] = None
    reset_password_template_name: Optional[str] = None
    reset_password_sender: Optional[str] = None
    reset_password_logo_url: Optional[str] = None
    reset_password_brand_name: Optional[str] = None
    reset_password_login_url: Optional[str] = None
    reset_password_site_base_url: Optional[str] = None
    reset_password_privacy_policy_url: Optional[str] = None
    reset_password_user_agreement_url: Optional[str] = None
    is_disabled: Optional[bool] = None
    is_editable: Optional[bool] = None
    secret_key: Optional[str] = None
    user_key: Optional[str] = None
    gigya_datacenter_url: Optional[str] = None
    gigya_passwordless_captcha_enabled: Optional[bool] = None
    gigya_passwordless_login_on_gift_redemption_enabled: Optional[bool] = None
    shared_secret: Optional[str] = None
    reset_password_landing_page_url: Optional[str] = None
    passwordless_magic_link_url: Optional[str] = None
    oauth_client_secret: Optional[str] = None
    token_expiration_time: Optional[int] = None
    password_reset_link_expiration_time_in_hours: Optional[int] = None
    deployment_host: Optional[str] = None
    password_minimal_length: Optional[int] = None
    password_has_mixed_case: Optional[bool] = None
    password_has_alphanumeric: Optional[bool] = None
    password_has_special_characters: Optional[bool] = None
    password_has_no_email: Optional[bool] = None
    use_captcha: Optional[bool] = None
    captcha_version: Optional[str] = None
    captcha_screen: Optional[str] = None
    captcha3_site_key: Optional[str] = None
    captcha3_secret_key: Optional[str] = None
    reset_password_captcha: Optional[bool] = None
    authentication_attempts_to_show_captcha: Optional[int] = None
    piano_id_captcha_settings: Optional[str] = None
    message_login_failed: Optional[str] = None
    social_callback_url: Optional[str] = None
    social_settings: Optional[str] = None
    piano_id_main_settings: Optional[str] = None
    require_first_and_last_names: Optional[bool] = None
    override_shared_secret_for_global_mode: Optional[bool] = None
    disable_custom_fields_user_mining: Optional[bool] = None
    disable_custom_fields_counters: Optional[bool] = None
    client_custom_fields_validation: Optional[bool] = None
    extend_expired_access_enabled: Optional[bool] = None
    passwordless_login_enabled: Optional[bool] = None
    redirect_uri_whitelist: Optional[List[str]] = None
    redirect_uri_whitelist_confirmed: Optional[bool] = None
    external_identity_providers: Optional[str] = None
    custom_fields: Optional['List[Dict[str, Any]]'] = None
    overridden_settings: Optional[str] = None


UserProviderConfiguration.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.export import Export


class PublisherExportCreateSubscriptionDetailsReportApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def create_subscription_detailed_report_tz(self, aid: str, export_name: str, q: str = None, search_new_subscriptions: bool = None, new_subscriptions_created_from: datetime = None, new_subscriptions_created_to: datetime = None, search_active_now_subscriptions: bool = None, active_now_subscriptions_statuses: List[str] = None, search_inactive_subscriptions: bool = None, inactive_subscriptions_statuses: List[str] = None, subscriptions_inactive_from: datetime = None, subscriptions_inactive_to: datetime = None, search_updated_subscriptions: bool = None, updated_subscriptions_statuses: List[str] = None, subscriptions_updated_from: datetime = None, subscriptions_updated_to: datetime = None, search_auto_renewing_subscriptions: bool = None, subscriptions_auto_renewing: bool = None, search_subscriptions_by_next_billing_date: bool = None, subscriptions_next_billing_date_from: datetime = None, subscriptions_next_billing_date_to: datetime = None, search_subscriptions_by_terms: bool = None, subscriptions_terms: List[str] = None, subscriptions_term_types: List[str] = None) -> ApiResponse[Export]:
        _url_path = '/api/v3/publisher/export/create/subscriptionDetailsReport/v2'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {

        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {
            'aid': _encode_parameter(aid),
            'export_name': _encode_parameter(export_name),
            'q': _encode_parameter(q),
            'search_new_subscriptions': _encode_parameter(search_new_subscriptions),
            'new_subscriptions_created_from': _encode_parameter(new_subscriptions_created_from),
            'new_subscriptions_created_to': _encode_parameter(new_subscriptions_created_to),
            'search_active_now_subscriptions': _encode_parameter(search_active_now_subscriptions),
            'active_now_subscriptions_statuses': _encode_parameter(active_now_subscriptions_statuses),
            'search_inactive_subscriptions': _encode_parameter(search_inactive_subscriptions),
            'inactive_subscriptions_statuses': _encode_parameter(inactive_subscriptions_statuses),
            'subscriptions_inactive_from': _encode_parameter(subscriptions_inactive_from),
            'subscriptions_inactive_to': _encode_parameter(subscriptions_inactive_to),
            'search_updated_subscriptions': _encode_parameter(search_updated_subscriptions),
            'updated_subscriptions_statuses': _encode_parameter(updated_subscriptions_statuses),
            'subscriptions_updated_from': _encode_parameter(subscriptions_updated_from),
            'subscriptions_updated_to': _encode_parameter(subscriptions_updated_to),
            'search_auto_renewing_subscriptions': _encode_parameter(search_auto_renewing_subscriptions),
            'subscriptions_auto_renewing': _encode_parameter(subscriptions_auto_renewing),
            'search_subscriptions_by_next_billing_date': _encode_parameter(search_subscriptions_by_next_billing_date),
            'subscriptions_next_billing_date_from': _encode_parameter(subscriptions_next_billing_date_from),
            'subscriptions_next_billing_date_to': _encode_parameter(subscriptions_next_billing_date_to),
            'search_subscriptions_by_terms': _encode_parameter(search_subscriptions_by_terms),
            'subscriptions_terms': _encode_parameter(subscriptions_terms),
            'subscriptions_term_types': _encode_parameter(subscriptions_term_types)
        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('POST',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, Export)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PublisherCustomFieldResponse(BaseModel):
    name: Optional[str] = None
    value: Optional[str] = None


PublisherCustomFieldResponse.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class PageInfo(BaseModel):
    offset: Optional[int] = None
    limit: Optional[int] = None
    total: Optional[int] = None
    limited: Optional[bool] = None
    end_page: Optional[int] = None
    begin_page: Optional[int] = None
    total_pages: Optional[int] = None
    has_next_page: Optional[bool] = None
    has_prev_page: Optional[bool] = None
    total_count_known: Optional[bool] = None


PageInfo.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class AuditChangedFieldDto(BaseModel):
    field_name: Optional[str] = None
    new_value: Optional[str] = None
    old_value: Optional[str] = None
    diff: Optional[str] = None


AuditChangedFieldDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from typing import List


class PpcCurrencyValidationDto(BaseModel):
    number_of_subscription: Optional[int] = None
    list_of_currencies: Optional[List[str]] = None
    can_be_deleted: Optional[bool] = None
    has_error: Optional[bool] = None


PpcCurrencyValidationDto.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class TermStats(BaseModel):
    pub_id: Optional[str] = None
    total_sale: Optional[str] = None
    total_sale_str: Optional[str] = None
    conversion: Optional[str] = None
    currency: Optional[str] = None


TermStats.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional
from pianosdk.publisher.models.target_eligible_term import TargetEligibleTerm
from pianosdk.publisher.models.target_ineligible_term import TargetIneligibleTerm
from pianosdk.publisher.models.target_upgrade_option import TargetUpgradeOption
from typing import List


class UpgradeTargets(BaseModel):
    already_configured_options: Optional['List[TargetUpgradeOption]'] = None
    eligible_options: Optional['List[TargetEligibleTerm]'] = None
    ineligible_options: Optional['List[TargetIneligibleTerm]'] = None


UpgradeTargets.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class ZuoraConnectConfiguration(BaseModel):
    email: Optional[str] = None
    zuora_api_token: Optional[str] = None


ZuoraConnectConfiguration.model_rebuild()



from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class EraseContractUser(BaseModel):
    contract_id: Optional[str] = None
    email: Optional[str] = None
    last_name: Optional[str] = None
    first_name: Optional[str] = None


EraseContractUser.model_rebuild()



from datetime import datetime
from io import StringIO
from typing import TextIO, Dict, List, Union

from pianosdk.api_response import ApiResponse
from pianosdk.base_api import BaseApi
from pianosdk.configuration import Configuration
from pianosdk.httpwrap import HttpCallBack
from pianosdk.utils import _json_deserialize, _encode_parameter
from pianosdk.publisher.models.webhook_event import WebhookEvent
from pianosdk.publisher.models.webhook_settings import WebhookSettings
from pianosdk.publisher.models.webhook_status import WebhookStatus


class PublisherWebhookApi(BaseApi):
    def __init__(self, config: Configuration, http_callback: HttpCallBack = None) -> None:
        super().__init__(config, http_callback)

    def events(self, aid: str, skip_deprecated_webhooks: bool = None) -> ApiResponse[List[str]]:
        _url_path = '/api/v3/publisher/webhook/events'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'skip_deprecated_webhooks': _encode_parameter(skip_deprecated_webhooks)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, str)
        return _result

    def get_event(self, webhook_id: str) -> ApiResponse[WebhookEvent]:
        _url_path = '/api/v3/publisher/webhook/get'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'webhook_id': _encode_parameter(webhook_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, WebhookEvent)
        return _result

    def get_settings(self, aid: str = None) -> ApiResponse[WebhookSettings]:
        _url_path = '/api/v3/publisher/webhook/settings'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, WebhookSettings)
        return _result

    def list(self, aid: str, offset: int = 0, limit: int = 100, order_by: str = 'create_date', order_direction: str = 'desc', status: str = None, keyword: str = None, event_type: List[str] = None, type: List[str] = None) -> ApiResponse[List[WebhookEvent]]:
        _url_path = '/api/v3/publisher/webhook/list'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid),
            'status': _encode_parameter(status),
            'keyword': _encode_parameter(keyword),
            'limit': _encode_parameter(limit),
            'offset': _encode_parameter(offset),
            'order_by': _encode_parameter(order_by),
            'order_direction': _encode_parameter(order_direction),
            'event_type': _encode_parameter(event_type),
            'type': _encode_parameter(type)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, WebhookEvent)
        return _result

    def skip(self, webhook_id: str) -> ApiResponse[WebhookEvent]:
        _url_path = '/api/v3/publisher/webhook/skip'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'webhook_id': _encode_parameter(webhook_id)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, WebhookEvent)
        return _result

    def status(self, aid: str) -> ApiResponse[WebhookStatus]:
        _url_path = '/api/v3/publisher/webhook/status'
        _query_url = self.config.get_base_url() + _url_path
        _query_parameters = {
            'aid': _encode_parameter(aid)
        }

        _headers = {
            'api_token': self.config.api_token,
            'Accept': 'application/json'
        }

        _parameters = {

        }

        _body = None
        _files = None

        _request = self.config.http_client.build_request('GET',
                                                         _query_url,
                                                         headers=_headers,
                                                         query_parameters=_query_parameters,
                                                         parameters=_parameters,
                                                         json=_body,
                                                         files=_files)
        _response = self._execute_request(_request)
        _result = _json_deserialize(_response, WebhookStatus)
        return _result




from datetime import date, datetime
from pydantic.main import BaseModel
from typing import Optional


class UserAccessDto(BaseModel):
    term_name: Optional[str] = None
    payment_billing_plan: Optional[str] = None
    billing_plan: Optional[str] = None
    image_url: Optional[str] = None
    resource_name: Optional[str] = None
    type: Optional[str] = None
    type_label: Optional[str] = None
    rid: Optional[str] = None
    term_id: Optional[str] = None
    create_date: Optional[str] = None
    expire_date: Optional[str] = None
    status: Optional[str] = None
    status_localized: Optional[str] = None
    access_id: Optional[str] = None


UserAccessDto.model_rebuild()



from pianosdk.user.api.user_api import UserApi
from pianosdk.user.api.user_access_api import UserAccessApi
from pianosdk.publisher.api.publisher_adblocker_api import PublisherAdblockerApi
from pianosdk.publisher.api.publisher_afc_configuration_api import PublisherAfcConfigurationApi
from pianosdk.publisher.api.publisher_app_api import PublisherAppApi
from pianosdk.publisher.api.publisher_app_api_token_api import PublisherAppApiTokenApi
from pianosdk.publisher.api.publisher_app_features_api import PublisherAppFeaturesApi
from pianosdk.publisher.api.publisher_consent_api import PublisherConsentApi
from pianosdk.publisher.api.publisher_consent_entry_api import PublisherConsentEntryApi
from pianosdk.publisher.api.publisher_conversion_api import PublisherConversionApi
from pianosdk.publisher.api.publisher_conversion_custom_api import PublisherConversionCustomApi
from pianosdk.publisher.api.publisher_conversion_data_api import PublisherConversionDataApi
from pianosdk.publisher.api.publisher_conversion_external_api import PublisherConversionExternalApi
from pianosdk.publisher.api.publisher_conversion_registration_api import PublisherConversionRegistrationApi
from pianosdk.publisher.api.publisher_email_confirmation_api import PublisherEmailConfirmationApi
from pianosdk.publisher.api.publisher_experience_metadata_api import PublisherExperienceMetadataApi
from pianosdk.publisher.api.publisher_export_api import PublisherExportApi
from pianosdk.publisher.api.publisher_export_create_api import PublisherExportCreateApi
from pianosdk.publisher.api.publisher_export_create_aam_api import PublisherExportCreateAamApi
from pianosdk.publisher.api.publisher_export_create_aam_monthly_api import PublisherExportCreateAamMonthlyApi
from pianosdk.publisher.api.publisher_export_create_access_report_export_api import PublisherExportCreateAccessReportExportApi
from pianosdk.publisher.api.publisher_export_create_subscription_details_report_api import PublisherExportCreateSubscriptionDetailsReportApi
from pianosdk.publisher.api.publisher_export_create_transactions_report_api import PublisherExportCreateTransactionsReportApi
from pianosdk.publisher.api.publisher_external_provider_payment_api import PublisherExternalProviderPaymentApi
from pianosdk.publisher.api.publisher_gdpr_api import PublisherGdprApi
from pianosdk.publisher.api.publisher_inquiry_api import PublisherInquiryApi
from pianosdk.publisher.api.publisher_licensing_contract_api import PublisherLicensingContractApi
from pianosdk.publisher.api.publisher_licensing_contract_domain_api import PublisherLicensingContractDomainApi
from pianosdk.publisher.api.publisher_licensing_contract_domain_contract_user_api import PublisherLicensingContractDomainContractUserApi
from pianosdk.publisher.api.publisher_licensing_contract_ip_range_api import PublisherLicensingContractIpRangeApi
from pianosdk.publisher.api.publisher_licensing_contract_periods_api import PublisherLicensingContractPeriodsApi
from pianosdk.publisher.api.publisher_licensing_contract_user_api import PublisherLicensingContractUserApi
from pianosdk.publisher.api.publisher_licensing_licensee_api import PublisherLicensingLicenseeApi
from pianosdk.publisher.api.publisher_licensing_notification_api import PublisherLicensingNotificationApi
from pianosdk.publisher.api.publisher_licensing_notification_rule_api import PublisherLicensingNotificationRuleApi
from pianosdk.publisher.api.publisher_licensing_schedule_api import PublisherLicensingScheduleApi
from pianosdk.publisher.api.publisher_licensing_schedule_contract_periods_api import PublisherLicensingScheduleContractPeriodsApi
from pianosdk.publisher.api.publisher_linked_term_api import PublisherLinkedTermApi
from pianosdk.publisher.api.publisher_linked_term_custom_field_api import PublisherLinkedTermCustomFieldApi
from pianosdk.publisher.api.publisher_offer_api import PublisherOfferApi
from pianosdk.publisher.api.publisher_offer_template_api import PublisherOfferTemplateApi
from pianosdk.publisher.api.publisher_offer_template_create_api import PublisherOfferTemplateCreateApi
from pianosdk.publisher.api.publisher_offer_template_inherited_api import PublisherOfferTemplateInheritedApi
from pianosdk.publisher.api.publisher_offer_template_update_api import PublisherOfferTemplateUpdateApi
from pianosdk.publisher.api.publisher_offer_template_variant_api import PublisherOfferTemplateVariantApi
from pianosdk.publisher.api.publisher_offer_term_api import PublisherOfferTermApi
from pianosdk.publisher.api.publisher_offer_term_offer_api import PublisherOfferTermOfferApi
from pianosdk.publisher.api.publisher_payment_api import PublisherPaymentApi
from pianosdk.publisher.api.publisher_payment_method_api import PublisherPaymentMethodApi
from pianosdk.publisher.api.publisher_payment_method_billing_address_api import PublisherPaymentMethodBillingAddressApi
from pianosdk.publisher.api.publisher_payment_method_gmo_api import PublisherPaymentMethodGmoApi
from pianosdk.publisher.api.publisher_payment_provider_configuration_api import PublisherPaymentProviderConfigurationApi
from pianosdk.publisher.api.publisher_promotion_api import PublisherPromotionApi
from pianosdk.publisher.api.publisher_promotion_code_api import PublisherPromotionCodeApi
from pianosdk.publisher.api.publisher_promotion_fixed_discount_api import PublisherPromotionFixedDiscountApi
from pianosdk.publisher.api.publisher_promotion_term_api import PublisherPromotionTermApi
from pianosdk.publisher.api.publisher_resource_api import PublisherResourceApi
from pianosdk.publisher.api.publisher_resource_bundle_api import PublisherResourceBundleApi
from pianosdk.publisher.api.publisher_resource_cross_app_api import PublisherResourceCrossAppApi
from pianosdk.publisher.api.publisher_resource_stats_api import PublisherResourceStatsApi
from pianosdk.publisher.api.publisher_resource_tag_api import PublisherResourceTagApi
from pianosdk.publisher.api.publisher_resource_user_api import PublisherResourceUserApi
from pianosdk.publisher.api.publisher_schedule_api import PublisherScheduleApi
from pianosdk.publisher.api.publisher_schedule_period_api import PublisherSchedulePeriodApi
from pianosdk.publisher.api.publisher_subscription_api import PublisherSubscriptionApi
from pianosdk.publisher.api.publisher_subscription_address_api import PublisherSubscriptionAddressApi
from pianosdk.publisher.api.publisher_subscription_cancel_api import PublisherSubscriptionCancelApi
from pianosdk.publisher.api.publisher_subscription_share_api import PublisherSubscriptionShareApi
from pianosdk.publisher.api.publisher_subscription_share_user_api import PublisherSubscriptionShareUserApi
from pianosdk.publisher.api.publisher_team_api import PublisherTeamApi
from pianosdk.publisher.api.publisher_term_api import PublisherTermApi
from pianosdk.publisher.api.publisher_term_change_api import PublisherTermChangeApi
from pianosdk.publisher.api.publisher_term_change_option_api import PublisherTermChangeOptionApi
from pianosdk.publisher.api.publisher_term_custom_api import PublisherTermCustomApi
from pianosdk.publisher.api.publisher_term_dynamic_api import PublisherTermDynamicApi
from pianosdk.publisher.api.publisher_term_external_api import PublisherTermExternalApi
from pianosdk.publisher.api.publisher_term_gift_api import PublisherTermGiftApi
from pianosdk.publisher.api.publisher_term_payment_api import PublisherTermPaymentApi
from pianosdk.publisher.api.publisher_term_registration_api import PublisherTermRegistrationApi
from pianosdk.publisher.api.publisher_term_stats_api import PublisherTermStatsApi
from pianosdk.publisher.api.publisher_test_api import PublisherTestApi
from pianosdk.publisher.api.publisher_user_api import PublisherUserApi
from pianosdk.publisher.api.publisher_user_access_api import PublisherUserAccessApi
from pianosdk.publisher.api.publisher_user_access_active_api import PublisherUserAccessActiveApi
from pianosdk.publisher.api.publisher_user_address_api import PublisherUserAddressApi
from pianosdk.publisher.api.publisher_user_app_api import PublisherUserAppApi
from pianosdk.publisher.api.publisher_user_billing_address_api import PublisherUserBillingAddressApi
from pianosdk.publisher.api.publisher_user_bulk_import_api import PublisherUserBulkImportApi
from pianosdk.publisher.api.publisher_user_email_api import PublisherUserEmailApi
from pianosdk.publisher.api.publisher_user_list_api import PublisherUserListApi
from pianosdk.publisher.api.publisher_user_note_api import PublisherUserNoteApi
from pianosdk.publisher.api.publisher_voucher_api import PublisherVoucherApi
from pianosdk.publisher.api.publisher_webhook_api import PublisherWebhookApi
from pianosdk.publisher.api.publisher_webhook_response_api import PublisherWebhookResponseApi
from pianosdk.publisher.api.publisher_webhook_settings_api import PublisherWebhookSettingsApi
from pianosdk.anon.api.access_api import AccessApi
from pianosdk.anon.api.access_token_api import AccessTokenApi
from pianosdk.anon.api.anon_assets_api import AnonAssetsApi
from pianosdk.anon.api.anon_country_list_api import AnonCountryListApi
from pianosdk.anon.api.anon_error_api import AnonErrorApi
from pianosdk.anon.api.anon_mobile_sdk_id_deployment_api import AnonMobileSdkIdDeploymentApi
from pianosdk.anon.api.anon_user_api import AnonUserApi
from pianosdk.anon.api.anon_user_disable_api import AnonUserDisableApi
from pianosdk.anon.api.conversion_api import ConversionApi
from pianosdk.anon.api.conversion_external_api import ConversionExternalApi
from pianosdk.anon.api.conversion_registration_api import ConversionRegistrationApi
from pianosdk.anon.api.email_confirmation_api import EmailConfirmationApi
from pianosdk.anon.api.exposure_api import ExposureApi
from pianosdk.anon.api.oauth_api import OauthApi
from pianosdk.anon.api.subscription_api import SubscriptionApi
from pianosdk.anon.api.swg_sync_api import SwgSyncApi
from pianosdk.id.api.identity_api import IdentityApi
from pianosdk.id.api.identity_oauth_api import IdentityOauthApi
from pianosdk.id.api.identity_token_api import IdentityTokenApi
from pianosdk.id.api.publisher_api import PublisherApi
from pianosdk.id.api.publisher_audit_api import PublisherAuditApi
from pianosdk.id.api.publisher_form_api import PublisherFormApi
from pianosdk.id.api.publisher_identity_api import PublisherIdentityApi
from pianosdk.id.api.publisher_identity_doi_api import PublisherIdentityDoiApi
from pianosdk.id.api.publisher_identity_session_api import PublisherIdentitySessionApi
from pianosdk.id.api.publisher_identity_set_api import PublisherIdentitySetApi
from pianosdk.id.api.publisher_import_custom_fields_api import PublisherImportCustomFieldsApi
from pianosdk.id.api.publisher_link_api import PublisherLinkApi
from pianosdk.id.api.publisher_login_api import PublisherLoginApi
from pianosdk.id.api.publisher_login_social_api import PublisherLoginSocialApi
from pianosdk.id.api.publisher_reset_api import PublisherResetApi
from pianosdk.id.api.publisher_social_api import PublisherSocialApi
from pianosdk.id.api.publisher_token_api import PublisherTokenApi
from pianosdk.id.api.publisher_userinfo_api import PublisherUserinfoApi
from pianosdk.id.api.publisher_users_api import PublisherUsersApi
import json
from datetime import datetime, timezone
from typing import Dict

from pianosdk import HttpCallBack
from pianosdk.configuration import Configuration
from pianosdk.utils import cached_property, _encrypt, _decrypt, _encode_parameter
from pianosdk.webhook_events import _event_types_mapping, Event


class Client:
    def __init__(self, timeout: int = 60, max_retries: int = 3, backoff_factor: int = 0,
                 api_host: str = 'production', api_token: str = 'TODO: Replace', private_key: str = 'TODO: Replace',
                 additional_headers: Dict = None, config: Configuration = None, http_callback: HttpCallBack = None) -> None:
        """
        API client. It will use provided `config` or will build configuration from other parameters
        :param timeout: The value to use for connection timeout
        :param max_retries: The number of times to retry failed endpoint call
        :param backoff_factor: A backoff factor to apply between attempts after the second try.
        :param api_host: API host. Use `production`, `sandbox` or custom url
        :param api_token: Api Token
        :param private_key: Private key
        :param additional_headers: Additional headers to add to each API request
        :param config: Prebuilt configuration
        :param http_callback: Http method call back to intercept raw requests or responses
        """
        if config is None:
            self.config = Configuration(timeout=timeout,
                                        max_retries=max_retries,
                                        backoff_factor=backoff_factor,
                                        api_host=api_host,
                                        api_token=api_token,
                                        private_key=private_key,
                                        additional_headers=additional_headers)
        else:
            self.config = config
        self.http_callback = http_callback

    def userref_create(self, uid: str, email: str, first_name: str = None, last_name: str = None,
                       create_date: datetime = None) -> str:
        """
        Build User reference
        :param uid: User id
        :param email: User's email
        :param first_name: User's first name
        :param last_name: User's last name
        :param create_date: User's create date
        :return: Encrypted user reference for Piano backend
        """
        data = {
            'uid': uid,
            'email': email,
            'first_name': first_name,
            'last_name': last_name,
            'create_date': create_date and str(_encode_parameter(create_date)),
            'timestamp': str(_encode_parameter(datetime.utcnow()))
        }
        data = {k: v for k, v in data.items() if v is not None}
        return _encrypt(self.config.private_key, json.dumps(data))

    def parse_webhook_data(self, value: str) -> Event:
        """
        Parse webhook event
        :param value: Encrypted webhook data
        :return: Event object
        """
        text = _decrypt(self.config.private_key, value)
        data = json.loads(text)
        event_type = data.get('type')
        if not event_type:
            raise ValueError("Event type missed, can't parse it")
        cls = _event_types_mapping.get(event_type)
        if not cls:
            raise ValueError(f'Unknown event type {event_type}')
        return cls(**data)

    @cached_property
    def user_api(self) -> UserApi:
        return UserApi(self.config, self.http_callback)

    @cached_property
    def user_access_api(self) -> UserAccessApi:
        return UserAccessApi(self.config, self.http_callback)

    @cached_property
    def publisher_adblocker_api(self) -> PublisherAdblockerApi:
        return PublisherAdblockerApi(self.config, self.http_callback)

    @cached_property
    def publisher_afc_configuration_api(self) -> PublisherAfcConfigurationApi:
        return PublisherAfcConfigurationApi(self.config, self.http_callback)

    @cached_property
    def publisher_app_api(self) -> PublisherAppApi:
        return PublisherAppApi(self.config, self.http_callback)

    @cached_property
    def publisher_app_api_token_api(self) -> PublisherAppApiTokenApi:
        return PublisherAppApiTokenApi(self.config, self.http_callback)

    @cached_property
    def publisher_app_features_api(self) -> PublisherAppFeaturesApi:
        return PublisherAppFeaturesApi(self.config, self.http_callback)

    @cached_property
    def publisher_consent_api(self) -> PublisherConsentApi:
        return PublisherConsentApi(self.config, self.http_callback)

    @cached_property
    def publisher_consent_entry_api(self) -> PublisherConsentEntryApi:
        return PublisherConsentEntryApi(self.config, self.http_callback)

    @cached_property
    def publisher_conversion_api(self) -> PublisherConversionApi:
        return PublisherConversionApi(self.config, self.http_callback)

    @cached_property
    def publisher_conversion_custom_api(self) -> PublisherConversionCustomApi:
        return PublisherConversionCustomApi(self.config, self.http_callback)

    @cached_property
    def publisher_conversion_data_api(self) -> PublisherConversionDataApi:
        return PublisherConversionDataApi(self.config, self.http_callback)

    @cached_property
    def publisher_conversion_external_api(self) -> PublisherConversionExternalApi:
        return PublisherConversionExternalApi(self.config, self.http_callback)

    @cached_property
    def publisher_conversion_registration_api(self) -> PublisherConversionRegistrationApi:
        return PublisherConversionRegistrationApi(self.config, self.http_callback)

    @cached_property
    def publisher_email_confirmation_api(self) -> PublisherEmailConfirmationApi:
        return PublisherEmailConfirmationApi(self.config, self.http_callback)

    @cached_property
    def publisher_experience_metadata_api(self) -> PublisherExperienceMetadataApi:
        return PublisherExperienceMetadataApi(self.config, self.http_callback)

    @cached_property
    def publisher_export_api(self) -> PublisherExportApi:
        return PublisherExportApi(self.config, self.http_callback)

    @cached_property
    def publisher_export_create_api(self) -> PublisherExportCreateApi:
        return PublisherExportCreateApi(self.config, self.http_callback)

    @cached_property
    def publisher_export_create_aam_api(self) -> PublisherExportCreateAamApi:
        return PublisherExportCreateAamApi(self.config, self.http_callback)

    @cached_property
    def publisher_export_create_aam_monthly_api(self) -> PublisherExportCreateAamMonthlyApi:
        return PublisherExportCreateAamMonthlyApi(self.config, self.http_callback)

    @cached_property
    def publisher_export_create_access_report_export_api(self) -> PublisherExportCreateAccessReportExportApi:
        return PublisherExportCreateAccessReportExportApi(self.config, self.http_callback)

    @cached_property
    def publisher_export_create_subscription_details_report_api(self) -> PublisherExportCreateSubscriptionDetailsReportApi:
        return PublisherExportCreateSubscriptionDetailsReportApi(self.config, self.http_callback)

    @cached_property
    def publisher_export_create_transactions_report_api(self) -> PublisherExportCreateTransactionsReportApi:
        return PublisherExportCreateTransactionsReportApi(self.config, self.http_callback)

    @cached_property
    def publisher_external_provider_payment_api(self) -> PublisherExternalProviderPaymentApi:
        return PublisherExternalProviderPaymentApi(self.config, self.http_callback)

    @cached_property
    def publisher_gdpr_api(self) -> PublisherGdprApi:
        return PublisherGdprApi(self.config, self.http_callback)

    @cached_property
    def publisher_inquiry_api(self) -> PublisherInquiryApi:
        return PublisherInquiryApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_contract_api(self) -> PublisherLicensingContractApi:
        return PublisherLicensingContractApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_contract_domain_api(self) -> PublisherLicensingContractDomainApi:
        return PublisherLicensingContractDomainApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_contract_domain_contract_user_api(self) -> PublisherLicensingContractDomainContractUserApi:
        return PublisherLicensingContractDomainContractUserApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_contract_ip_range_api(self) -> PublisherLicensingContractIpRangeApi:
        return PublisherLicensingContractIpRangeApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_contract_periods_api(self) -> PublisherLicensingContractPeriodsApi:
        return PublisherLicensingContractPeriodsApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_contract_user_api(self) -> PublisherLicensingContractUserApi:
        return PublisherLicensingContractUserApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_licensee_api(self) -> PublisherLicensingLicenseeApi:
        return PublisherLicensingLicenseeApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_notification_api(self) -> PublisherLicensingNotificationApi:
        return PublisherLicensingNotificationApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_notification_rule_api(self) -> PublisherLicensingNotificationRuleApi:
        return PublisherLicensingNotificationRuleApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_schedule_api(self) -> PublisherLicensingScheduleApi:
        return PublisherLicensingScheduleApi(self.config, self.http_callback)

    @cached_property
    def publisher_licensing_schedule_contract_periods_api(self) -> PublisherLicensingScheduleContractPeriodsApi:
        return PublisherLicensingScheduleContractPeriodsApi(self.config, self.http_callback)

    @cached_property
    def publisher_linked_term_api(self) -> PublisherLinkedTermApi:
        return PublisherLinkedTermApi(self.config, self.http_callback)

    @cached_property
    def publisher_linked_term_custom_field_api(self) -> PublisherLinkedTermCustomFieldApi:
        return PublisherLinkedTermCustomFieldApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_api(self) -> PublisherOfferApi:
        return PublisherOfferApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_template_api(self) -> PublisherOfferTemplateApi:
        return PublisherOfferTemplateApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_template_create_api(self) -> PublisherOfferTemplateCreateApi:
        return PublisherOfferTemplateCreateApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_template_inherited_api(self) -> PublisherOfferTemplateInheritedApi:
        return PublisherOfferTemplateInheritedApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_template_update_api(self) -> PublisherOfferTemplateUpdateApi:
        return PublisherOfferTemplateUpdateApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_template_variant_api(self) -> PublisherOfferTemplateVariantApi:
        return PublisherOfferTemplateVariantApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_term_api(self) -> PublisherOfferTermApi:
        return PublisherOfferTermApi(self.config, self.http_callback)

    @cached_property
    def publisher_offer_term_offer_api(self) -> PublisherOfferTermOfferApi:
        return PublisherOfferTermOfferApi(self.config, self.http_callback)

    @cached_property
    def publisher_payment_api(self) -> PublisherPaymentApi:
        return PublisherPaymentApi(self.config, self.http_callback)

    @cached_property
    def publisher_payment_method_api(self) -> PublisherPaymentMethodApi:
        return PublisherPaymentMethodApi(self.config, self.http_callback)

    @cached_property
    def publisher_payment_method_billing_address_api(self) -> PublisherPaymentMethodBillingAddressApi:
        return PublisherPaymentMethodBillingAddressApi(self.config, self.http_callback)

    @cached_property
    def publisher_payment_method_gmo_api(self) -> PublisherPaymentMethodGmoApi:
        return PublisherPaymentMethodGmoApi(self.config, self.http_callback)

    @cached_property
    def publisher_payment_provider_configuration_api(self) -> PublisherPaymentProviderConfigurationApi:
        return PublisherPaymentProviderConfigurationApi(self.config, self.http_callback)

    @cached_property
    def publisher_promotion_api(self) -> PublisherPromotionApi:
        return PublisherPromotionApi(self.config, self.http_callback)

    @cached_property
    def publisher_promotion_code_api(self) -> PublisherPromotionCodeApi:
        return PublisherPromotionCodeApi(self.config, self.http_callback)

    @cached_property
    def publisher_promotion_fixed_discount_api(self) -> PublisherPromotionFixedDiscountApi:
        return PublisherPromotionFixedDiscountApi(self.config, self.http_callback)

    @cached_property
    def publisher_promotion_term_api(self) -> PublisherPromotionTermApi:
        return PublisherPromotionTermApi(self.config, self.http_callback)

    @cached_property
    def publisher_resource_api(self) -> PublisherResourceApi:
        return PublisherResourceApi(self.config, self.http_callback)

    @cached_property
    def publisher_resource_bundle_api(self) -> PublisherResourceBundleApi:
        return PublisherResourceBundleApi(self.config, self.http_callback)

    @cached_property
    def publisher_resource_cross_app_api(self) -> PublisherResourceCrossAppApi:
        return PublisherResourceCrossAppApi(self.config, self.http_callback)

    @cached_property
    def publisher_resource_stats_api(self) -> PublisherResourceStatsApi:
        return PublisherResourceStatsApi(self.config, self.http_callback)

    @cached_property
    def publisher_resource_tag_api(self) -> PublisherResourceTagApi:
        return PublisherResourceTagApi(self.config, self.http_callback)

    @cached_property
    def publisher_resource_user_api(self) -> PublisherResourceUserApi:
        return PublisherResourceUserApi(self.config, self.http_callback)

    @cached_property
    def publisher_schedule_api(self) -> PublisherScheduleApi:
        return PublisherScheduleApi(self.config, self.http_callback)

    @cached_property
    def publisher_schedule_period_api(self) -> PublisherSchedulePeriodApi:
        return PublisherSchedulePeriodApi(self.config, self.http_callback)

    @cached_property
    def publisher_subscription_api(self) -> PublisherSubscriptionApi:
        return PublisherSubscriptionApi(self.config, self.http_callback)

    @cached_property
    def publisher_subscription_address_api(self) -> PublisherSubscriptionAddressApi:
        return PublisherSubscriptionAddressApi(self.config, self.http_callback)

    @cached_property
    def publisher_subscription_cancel_api(self) -> PublisherSubscriptionCancelApi:
        return PublisherSubscriptionCancelApi(self.config, self.http_callback)

    @cached_property
    def publisher_subscription_share_api(self) -> PublisherSubscriptionShareApi:
        return PublisherSubscriptionShareApi(self.config, self.http_callback)

    @cached_property
    def publisher_subscription_share_user_api(self) -> PublisherSubscriptionShareUserApi:
        return PublisherSubscriptionShareUserApi(self.config, self.http_callback)

    @cached_property
    def publisher_team_api(self) -> PublisherTeamApi:
        return PublisherTeamApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_api(self) -> PublisherTermApi:
        return PublisherTermApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_change_api(self) -> PublisherTermChangeApi:
        return PublisherTermChangeApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_change_option_api(self) -> PublisherTermChangeOptionApi:
        return PublisherTermChangeOptionApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_custom_api(self) -> PublisherTermCustomApi:
        return PublisherTermCustomApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_dynamic_api(self) -> PublisherTermDynamicApi:
        return PublisherTermDynamicApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_external_api(self) -> PublisherTermExternalApi:
        return PublisherTermExternalApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_gift_api(self) -> PublisherTermGiftApi:
        return PublisherTermGiftApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_payment_api(self) -> PublisherTermPaymentApi:
        return PublisherTermPaymentApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_registration_api(self) -> PublisherTermRegistrationApi:
        return PublisherTermRegistrationApi(self.config, self.http_callback)

    @cached_property
    def publisher_term_stats_api(self) -> PublisherTermStatsApi:
        return PublisherTermStatsApi(self.config, self.http_callback)

    @cached_property
    def publisher_test_api(self) -> PublisherTestApi:
        return PublisherTestApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_api(self) -> PublisherUserApi:
        return PublisherUserApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_access_api(self) -> PublisherUserAccessApi:
        return PublisherUserAccessApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_access_active_api(self) -> PublisherUserAccessActiveApi:
        return PublisherUserAccessActiveApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_address_api(self) -> PublisherUserAddressApi:
        return PublisherUserAddressApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_app_api(self) -> PublisherUserAppApi:
        return PublisherUserAppApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_billing_address_api(self) -> PublisherUserBillingAddressApi:
        return PublisherUserBillingAddressApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_bulk_import_api(self) -> PublisherUserBulkImportApi:
        return PublisherUserBulkImportApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_email_api(self) -> PublisherUserEmailApi:
        return PublisherUserEmailApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_list_api(self) -> PublisherUserListApi:
        return PublisherUserListApi(self.config, self.http_callback)

    @cached_property
    def publisher_user_note_api(self) -> PublisherUserNoteApi:
        return PublisherUserNoteApi(self.config, self.http_callback)

    @cached_property
    def publisher_voucher_api(self) -> PublisherVoucherApi:
        return PublisherVoucherApi(self.config, self.http_callback)

    @cached_property
    def publisher_webhook_api(self) -> PublisherWebhookApi:
        return PublisherWebhookApi(self.config, self.http_callback)

    @cached_property
    def publisher_webhook_response_api(self) -> PublisherWebhookResponseApi:
        return PublisherWebhookResponseApi(self.config, self.http_callback)

    @cached_property
    def publisher_webhook_settings_api(self) -> PublisherWebhookSettingsApi:
        return PublisherWebhookSettingsApi(self.config, self.http_callback)

    @cached_property
    def access_api(self) -> AccessApi:
        return AccessApi(self.config, self.http_callback)

    @cached_property
    def access_token_api(self) -> AccessTokenApi:
        return AccessTokenApi(self.config, self.http_callback)

    @cached_property
    def anon_assets_api(self) -> AnonAssetsApi:
        return AnonAssetsApi(self.config, self.http_callback)

    @cached_property
    def anon_country_list_api(self) -> AnonCountryListApi:
        return AnonCountryListApi(self.config, self.http_callback)

    @cached_property
    def anon_error_api(self) -> AnonErrorApi:
        return AnonErrorApi(self.config, self.http_callback)

    @cached_property
    def anon_mobile_sdk_id_deployment_api(self) -> AnonMobileSdkIdDeploymentApi:
        return AnonMobileSdkIdDeploymentApi(self.config, self.http_callback)

    @cached_property
    def anon_user_api(self) -> AnonUserApi:
        return AnonUserApi(self.config, self.http_callback)

    @cached_property
    def anon_user_disable_api(self) -> AnonUserDisableApi:
        return AnonUserDisableApi(self.config, self.http_callback)

    @cached_property
    def conversion_api(self) -> ConversionApi:
        return ConversionApi(self.config, self.http_callback)

    @cached_property
    def conversion_external_api(self) -> ConversionExternalApi:
        return ConversionExternalApi(self.config, self.http_callback)

    @cached_property
    def conversion_registration_api(self) -> ConversionRegistrationApi:
        return ConversionRegistrationApi(self.config, self.http_callback)

    @cached_property
    def email_confirmation_api(self) -> EmailConfirmationApi:
        return EmailConfirmationApi(self.config, self.http_callback)

    @cached_property
    def exposure_api(self) -> ExposureApi:
        return ExposureApi(self.config, self.http_callback)

    @cached_property
    def oauth_api(self) -> OauthApi:
        return OauthApi(self.config, self.http_callback)

    @cached_property
    def subscription_api(self) -> SubscriptionApi:
        return SubscriptionApi(self.config, self.http_callback)

    @cached_property
    def swg_sync_api(self) -> SwgSyncApi:
        return SwgSyncApi(self.config, self.http_callback)

    @cached_property
    def identity_api(self) -> IdentityApi:
        return IdentityApi(self.config, self.http_callback)

    @cached_property
    def identity_oauth_api(self) -> IdentityOauthApi:
        return IdentityOauthApi(self.config, self.http_callback)

    @cached_property
    def identity_token_api(self) -> IdentityTokenApi:
        return IdentityTokenApi(self.config, self.http_callback)

    @cached_property
    def publisher_api(self) -> PublisherApi:
        return PublisherApi(self.config, self.http_callback)

    @cached_property
    def publisher_audit_api(self) -> PublisherAuditApi:
        return PublisherAuditApi(self.config, self.http_callback)

    @cached_property
    def publisher_form_api(self) -> PublisherFormApi:
        return PublisherFormApi(self.config, self.http_callback)

    @cached_property
    def publisher_identity_api(self) -> PublisherIdentityApi:
        return PublisherIdentityApi(self.config, self.http_callback)

    @cached_property
    def publisher_identity_doi_api(self) -> PublisherIdentityDoiApi:
        return PublisherIdentityDoiApi(self.config, self.http_callback)

    @cached_property
    def publisher_identity_session_api(self) -> PublisherIdentitySessionApi:
        return PublisherIdentitySessionApi(self.config, self.http_callback)

    @cached_property
    def publisher_identity_set_api(self) -> PublisherIdentitySetApi:
        return PublisherIdentitySetApi(self.config, self.http_callback)

    @cached_property
    def publisher_import_custom_fields_api(self) -> PublisherImportCustomFieldsApi:
        return PublisherImportCustomFieldsApi(self.config, self.http_callback)

    @cached_property
    def publisher_link_api(self) -> PublisherLinkApi:
        return PublisherLinkApi(self.config, self.http_callback)

    @cached_property
    def publisher_login_api(self) -> PublisherLoginApi:
        return PublisherLoginApi(self.config, self.http_callback)

    @cached_property
    def publisher_login_social_api(self) -> PublisherLoginSocialApi:
        return PublisherLoginSocialApi(self.config, self.http_callback)

    @cached_property
    def publisher_reset_api(self) -> PublisherResetApi:
        return PublisherResetApi(self.config, self.http_callback)

    @cached_property
    def publisher_social_api(self) -> PublisherSocialApi:
        return PublisherSocialApi(self.config, self.http_callback)

    @cached_property
    def publisher_token_api(self) -> PublisherTokenApi:
        return PublisherTokenApi(self.config, self.http_callback)

    @cached_property
    def publisher_userinfo_api(self) -> PublisherUserinfoApi:
        return PublisherUserinfoApi(self.config, self.http_callback)

    @cached_property
    def publisher_users_api(self) -> PublisherUsersApi:
        return PublisherUsersApi(self.config, self.http_callback)

